<?php
/**
 * Exception thrown by the Phonekey Webservice
 * @version 0.1
 *
 */
class PhoneKeyServiceException extends Exception{

	/*
	 * Error while decrypting a request
	 */
	const EX_MALFORMED_REQUEST = 0;
	const EX_BAD_API_KEY = 1;
	const EX_CLIENT_PUBLIC_KEY_NOT_FOUND = 2;
	const EX_BAD_DATA2_CHUNKING = 3;
	const EX_BAD_SDATA2_BASE64_DECODING = 4;
	const EX_BAD_SKEY2_BASE64_DECODING = 5;
	const EX_BAD_SKEY2_DECRYPTING = 6;
	const EX_BAD_SDATA2_DECRYPTING = 7;
	const EX_BAD_DATA1_CHUNKING = 8;
	const EX_BAD_SDATA1_BASE64_DECODING = 9;
	const EX_BAD_SKEY1_BASE64_DECODING = 10;
	const EX_BAD_SKEY1_DECRYPTING = 11;
	const EX_BAD_SDATA1_DECRYPTING = 12;
	const EX_BAD_DATA1_INFLATING = 13;

	/*
	 * Error while crytping a request
	 */
	const EX_JSON_DEFLATIN_ERROR = 14;
	const EX_RANDOM_BYTES_FAILED = 15;
	const EX_BAD_KEY1_ENCRYPTING = 16;
	const EX_BAD_KEY2_ENCRYPTING = 17;
	const EX_BAD_SKEY1_BASE64_ENCODING = 18;
	const EX_BAD_SKEY2_BASE64_ENCODING = 19;
	const EX_BAD_DATA1_ENCRYPTING = 20;
	const EX_BAD_SDATA1_BASE64_ENCODING = 21;
	const EX_BAD_DATA2_CRYPTING = 22;
	const EX_BAD_SDATA2_BASE64_ENCODING = 23;

	/*
	 * Webservice error
	 */
	const EX_REQUEST_LIMIT_REACHED = 24;
	const EX_TOKEN_NOT_FOUND = 25;
	const EX_BAD_RESPONSE_FORMAT = 26;
	const EX_WEBSERVICE_SPECIFIC_ERROR = 27;

	// Unknown error
	const EX_UNKNOWN_ERROR = 28;


	/*
	 * Association between error code and error explication + equivalent response code
	 */
	private static $EX_MSGS = array(
		// Decryption errors
		array("malformed request", 400),
		array("bad api key", 401),
		array("public key not found", 401),
		array("bad data2 chunking", 500),
		array("bad sdata2 base64 encoding", 500),
		array("bad skey2 base64 decoding", 500),
		array("bad skey2 decrypting", 500),
		array("bad sdata2 decrypting", 500),
		array("bad data1 chunking", 500),
		array("bad sdata1 base64 decoding", 500),
		array("bad skey1 base64 decoding", 500),
		array("bad skey1 decrypting", 500),
		array("bad data1 decrypting", 500),
		array("bad data1 inflating", 500),

		// Crypting errors
		array("unable to build json", 500),
		array("unable to generate a random symetric key", 500),
		array("unable to crypt key1", 500),
		array("unable to crypt key2", 500),
		array("unable to base 64 encode key1", 500),
		array("unable to base 64 encode key2", 500),
		array("unable to encrypt data1", 500),
		array("unable to base 64 encode data1", 500),
		array("unable to encrypt data 2", 500),
		array("unable to base 64 encode data 2", 500),

		// Communication errors
		array("OTP limit reached", 509),
		array("Token not found", 404),
		array("Bad response from server", 500),
		array("Webservice error", 500),

		// Specific error
		array("Unknown/non standard error", 500),
	);

	// The http code to return depending of the error
	private $http_code;

	/**
	 * Construct a PhoneKeyServiceException from an error code.
	 * If a non existing error code is provided, it is strongly recommended to give a $message_complement
	 * @param number $error_code
	 * @param string $message_complement
	 */
	public function __construct($error_code, $message_complement = ""){
		if(isset(PhoneKeyServiceException::$EX_MSGS[$error_code])){
			parent::__construct(PhoneKeyServiceException::$EX_MSGS[$error_code][0].($message_complement === "" ? "" : " ".$message_complement), $error_code);
			$this->http_code = PhoneKeyServiceException::$EX_MSGS[$error_code][1];
		}else{
			if($message_complement === ""){
				parent::__construct(PhoneKeyServiceException::$EX_MSGS[EX_WEBSERVICE_SPECIFIC_ERROR], EX_WEBSERVICE_SPECIFIC_ERROR);
			}else{
				parent::__construct(PhoneKeyServiceException::$EX_MSGS[EX_WEBSERVICE_SPECIFIC_ERROR]." : ".$message_complement, EX_WEBSERVICE_SPECIFIC_ERROR);
			}
			$this->http_code = 500;
		}
	}

	/**
	 * Not used by the API
	 * @return number
	 */
	public function getHttpCode(){
		return $this->http_code;
	}
}