<?php
/**
 * File usage_example.php : must be launched in a web browser via a http+php5 server
 * This file aims is to show you how to use the Phonekey API through 2 examples in your favorite browser (javascript must be enabled).
 * The code is documented to help you understand the API mecanism
 * What you need to run it :
 * 	PhoneKeyAPI.class.php
 *  PhoneKeyServiceException.class.php
 *  config_file.conf (or the config file you can generate with config_help.php)
 * @version 0.1
 *
 */

###
### 1. Initiating PhoneKey Webservice
###

// We check if all is ok to run the API
// You need to provide a pth to your config file, see the config_help.php to generate it
$errors = PhoneKeyAPI::checkEnvironment("config_file.conf");
if(count($errors) > 0){
	display_headers("Error");
	echo "<h3>Ooops...</h3><p>You will not be able to instanciate a PhoneKeyAPI object until these errors are corrected :<ul>".implode("", array_map(function($e){return "<li>".$e."</li>";}, $errors))."</ul>";
	display_footers();
	exit;
}

// We can build a PhoneKey API object : requires a config file
$api = new PhoneKeyAPI("config_file.conf");
// Testing service, it returns true if it's ok, other wise a string containing the error
if(($r = $api->checkService()) !== true){
	display_headers("Error");
	echo "<h3>Ooops...</h3><p>An error has occured while esthablishing a link to the webservice : '".$r."'";
	display_footers();
	exit;
}

###
### 2. Starting session and loading token from the last session
###

// Starting session
session_start();

// The session token variable name
$token_variable_name = "tok";

// Loading an existing token
$token = null;
if(isset($_SESSION[$token_variable_name]) && is_array(($_SESSION[$token_variable_name]))){
	$token = $api->getStatus($_SESSION[$token_variable_name]["token"]);
}

/*
 *	Here comes the structure of a token, that is what is returned by the
 *   requestLocalAuthentication
 *   requestRemoteAuthentication
 *   getStatus
 *  functions of the API :
 *	it is a PHP array(
		"token" 				<string>		the code of the token, used to retrieve informations from the webservice

		"type" 					<string>		("local"|"remote") : it determines the availability of some following properties

		"creationDate" 			<int>			the birthdate of the token as a GMT unix timestamp

		"duration" 				<int>			the time valilidy, in seconds, of the token. If duration = 0 it indicates that the token was cancelled (by the creation of a new token with the same phone number or by the client)

		"timeleft" 				<int>			number of second remaining before the token expires. It is calculated by the webservice so it is very accurate

		"validated"				<int|null>		if the token has been validated then it contains the date of validation (GMT unix timestamp), null if not validated

		"clientPhoneNumber"		<string|null>	null if type=="remote", otherwise it contains the phone number listened by the token (format : +00000000000)

		"phoneKeyPhoneNumber" 	<string|null>	null if type=="remote", otherwise it contains the phone number that is awaiting the clientPhoneNumber SMS

		"phoneCode" 			<string|null>	null if type=="remote", otherwise it contains the code that must be sent to the phoneKeyPhoneNumber by clientPhoneNumber

		"messagesCount" 		<int|null>		null if type=="remote", otherwise it tells how many messages were received from the clientPhoneNumber in order to validate the token

		"onSuccessURL" 			<string|null>	null if type="local", otherwise it contains the URL used to redirect the client after a successful authentication

		"onAbortURL" 			<string|null>	null if type="local", otherwise it contains the URL used to redirect the client if he wants to abort the authentication

		"onTimeOutURL" 			<string|null>	null if type="local", otherwise it contains the URL used to redirect the client if his authentication times out

		"remoteURL" 			<string|null>	null if type="local", otherwise it contains the URL on which the client must be redirected in order to proceed to a remote authentication
);
*/

###
### 3. Processing request to usage_exemple.php
###

// The possible requests are :
//  - newLocal : creates a new token for a local authentication (works if the phone number has been posted). If a token already exists, the operation is aborted.
//  - newRemote : creates a new token for a remote authentication; If a token already exists, the operation is aborted
//  - cancelCurrent : cancel the current token, the webservice is notfiied and cancel the token if it has not been validated or has not timed out
//  - forgetCurrent : reset the token stored in session, it doesn't send a cancel signal to the webservice to cancel the token.
// By default, if no particular request was submitted, the state of the last token is displayed and updated every 10 seconds
if(!empty($_GET["q"])){
	$q = $_GET["q"];

	// Creating a new local token
	if($q == "newLocal" && !empty($_POST["phonenumber"])){
		if($token == null){
			try{
				$_SESSION[$token_variable_name] = $api->requestLocalAuthentication($_POST["phonenumber"]);
				header("location: ".$_SERVER["PHP_SELF"]);
			}catch(PhoneKeyServiceException $e){
				display_headers("Error");
				display_phonekeyapi_exception($e);
				display_index_link();
				display_footers();
			}
		}else{
			header("location: ".$_SERVER["PHP_SELF"]);
		}
	}
	// Creating a new remote token
	else if($q == "newRemote"){
		try{
			// The base of the url for redirection
			$urlBase = "http://".$_SERVER["SERVER_NAME"].$_SERVER["PHP_SELF"]."?q=";
			$_SESSION[$token_variable_name] = $api->requestRemoteAuthentication($urlBase."remoteSuccess", $urlBase."remoteAbort", $urlBase."remoteTimeOut");
			header("location: ".$_SERVER["PHP_SELF"]);
		}catch(PhoneKeyServiceException $e){
			display_headers("Error");
			display_phonekeyapi_exception($e);
			display_index_link();
			display_footers();
		}
	}else if($q == "newRemoteFilled" && !empty($_POST["phonenumber"])){
		try{
			// The base of the url for redirection
			$urlBase = "http://".$_SERVER["SERVER_NAME"].$_SERVER["PHP_SELF"]."?q=";
			$_SESSION[$token_variable_name] = $api->requestRemoteAuthenticationWithPhoneNumber($_POST["phonenumber"], $urlBase."remoteSuccess", $urlBase."remoteAbort", $urlBase."remoteTimeOut");
			header("location: ".$_SERVER["PHP_SELF"]);
		}catch(PhoneKeyServiceException $e){
			display_headers("Error");
			display_phonekeyapi_exception($e);
			display_index_link();
			display_footers();
		}
	}
	// Unset the current token from the PhP Session
	else if($q == "forgetCurrent"){
		// Destroying the current token
		unset($_SESSION[$token_variable_name]);
		// Redirecting to the index
		header("location: ".$_SERVER["SCRIPT_NAME"]);
	}
	// A remote token has been validated
	else if($q == "remoteSuccess"){
		display_headers("Token success");
		echo '<span style="color: green;">The token has beed validated</span>';
		display_index_link();
		display_footers();
	}
	// A remote token has been cancelled
	else if($q == "remoteAbort"){
		display_headers("Token abort");
		echo '<span style="color: red;">The token has been canceled</span>';
		display_index_link();
		display_footers();
	}
	// A remote token has timed out
	else if($q == "remoteTimeOut"){
		display_headers("Token timeout");
		echo '<span style="color: red;">The token has timedout</span>';
		display_index_link();
		display_footers();
	// If nothing matches
	}else{
		header("location: ".$_SERVER["PHP_SELF"]);
	}
}else{
	// If a token is pending
	if($token != null){
		display_headers("Pending token", 10);
		display_token_infos($token);
		display_forget_token();
		display_footers();
	}
	// If no token is pending
	else{
		display_headers("Index");
		echo "<p>Currently, there is no token in your session.</p><h3>You can</h3>";
		display_newlocal_token();
		echo "<h3>or</h3>";
		display_newremote_token();
		echo "<h3>or</h3>";
		display_newremote_token_filled();
		display_footers();
	}
}

###
### FUNCTION USED TO DISPLAY INFORMATIONS TO THE BROWSER
###

// Displays the page headers
// $title : the title to give to the page
// $_auto_refresh : the amount of seconds to wait before auto-refreshing the page. 0 = no auto-refresh
function display_headers($title, $auto_refresh = 0){
	echo '<html lang="en">
<head>
<meta charset="utf-8">
<meta http-equiv="X-UA-Compatible" content="IE=edge">
<meta name="viewport" content="width=device-width, initial-scale=1">'.
($auto_refresh > 0 ? '<meta http-equiv="refresh" content="'.$auto_refresh.'" >' : '').
'<title>Phonekey API | '.$title.'</title>
</head>
<body>
<h2>Phonekey API : basic usage example - '.$title.'</h2>';
}

// Displays a link to create a new remote token without a phone number
function display_newremote_token_filled(){
	echo '<a href="'.$_SERVER["SCRIPT_NAME"].'?q=newRemote">create a new remote token without phone number</a><br />';
}

// Displays a form to create a new remote token without phone number
function display_newremote_token(){
	echo '<p>Create a new remote token with a phone number :</p><form action="?q=newRemoteFilled" method="POST">
	<input name="phonenumber" type="text" />
	<input type="Submit" />
</form>';
}

// Displays a form to create a new local token with a phone number
function display_newlocal_token(){
	echo '<p>Create a new local token with a phone number :</p><form action="?q=newLocal" method="POST">
<input name="phonenumber" type="text" />
<input type="Submit" />
</form>';
}

// Displays a link to forget the token stored in session
function display_forget_token(){
	echo '<a href="'.$_SERVER["SCRIPT_NAME"].'?q=forgetCurrent" onCLick="return confirm(\'Do you want to forget this token ?\nIt will remove it from your PHP session\');">forget this token</a> (unset the current token from the PhP Session)<br />';
}

// Shows informations about a token
function display_token_infos($token){

	// Some explainations including the type of data waited
	$map = array(
		"token" => array("The <b>ID</b> of the token.", "str"),
		"type" => array('The <b>type of the token</b>, can be a "remote" or a "local" token', "str"),
		"creationDate" => array("The UNIX timestamp of the token <b>creation date</b>", "date"),
		"duration" => array("The <b>time validity</b> of the token in seconds. If the value is 0, it means that the token as been cancelled", "str"),
		"timeleft" => array("The real time number of <b>seconds remaining</b> before the token times out", "str"),
		"validated" => array("Contains the UNIX <b>timestamp of the token validation</b>, null if the token is not yet validated", "date"),
		"clientPhoneNumber" => array('If the token is a "local" token, it contains the phone number that must <b>send the sms</b>, null otherwise', "str"),
		"phoneKeyPhoneNumber" => array('If the token is a "local" token, it contains the phone number that is <b>avaiting the sms</b>, null otherwise', "str"),
		"phoneCode"	=> array('If the token is a "local" token, it contains the phone <b>code to be sent by sms</b>', 'str'),
		"messagesCount"	=> array('If the token is a "local" token, it contains the <b>number of attempt</b> of authentication received by sms, null otherwise', 'str'),
		"onSuccessURL" => array('If the token is a "remote" token, it contains the URL provided to redirect client on authentication <b>success</b>, null otherwise', 'url'),
		"onAbortURL" => array('If the token is a "remote" token, it contains the URL provided to redirect client on authentication <b>abort</b>, null otherwise', 'url'),
		"onTimeOutURL" => array('If the token is a "remote" token, it contains the URL provided to redirect client on authentication <b>time out</b>, null otherwise', 'url'),
		"remoteURL" => array('If the token is a "remote" token, it contains the <b>URL on which the client can authenticate</b>, null otherwise', 'url'),
	);
	// Building an html table from the token array fields
	$style = ' style="border: 1px solid black;"';
	echo '<table '.$style.'><tr style="text-weight: bold;"><th'.$style.'>Key</th><th'.$style.'>Value</th><th'.$style.'>Explaination</th></tr>';
	foreach($token as $k => $v){
		echo '<tr><td'.$style.'>'.$k.'</td><td'.$style.'>';
		// formatting the data according to their type
		if($v === null)
			echo '<b><i>NULL</i></b>';
		else if($map[$k][1] === "url")
			echo '<a href="'.$v.'" target="_blank">'.$v.'</a>';
		else if($map[$k][1] === "date")
			echo date("Y-m-d H:i:s", $v);
		else
			echo $v;
		echo '</td><td'.$style.'>'.$map[$k][0].'</td></tr>';
	}
	echo "</table>";
}

function display_index_link(){
	echo '<p>Go to the index : <a href="'.$_SERVER["SCRIPT_NAME"].'">index</a><br /></p>';
}

// Shows inforamtions about a PhoneKeyServiceException
// $e the exception
function display_phonekeyapi_exception(PhoneKeyServiceException $e){
	echo "<h3>Ooops...</h3><p>An error has occured : <b>".$e->getMessage()."</b></p>";
}

// Displays the page footer
function display_footers(){
	echo "</body></html>";
}

// Class autoloading
function __autoload($classe){
	if(file_exists($classe.".class.php")){
		require_once($classe.".class.php");
	}else{
		debug_print_backtrace();
		die("Unable to load '".$classe."'");
	}
}